"""Simplified python code documentation."""
from .doc import (
    append_doc,
    append_plot,
    append_str,
    array_table,
    dict_to_table,
    insert_doc,
    insert_eq,
    insert_latex,
    insert_latex_eq,
    insert_str,
    table,
)

__all__ = [
    "append_doc",
    "append_plot",
    "append_str",
    "insert_doc",
    "array_table",
    "dict_to_table",
    "insert_str",
    "insert_eq",
    "table",
    "insert_latex",
    "insert_latex_eq",
]
