"""Simplified internal data parsing and transforming."""

from .data import __data_split, data_kwargs, filtered_data_split, flatmesh

__all__ = ["filtered_data_split", "__data_split", "data_kwargs", "flatmesh"]
