"""Simplified python debuging."""

from .debug import (
    file,
    file1,
    get_line_number_file,
    msg,
    msg1,
    once,
    table,
    table_flush_header,
    table_flush_line,
)

__all__ = [
    "msg",
    "once",
    "msg1",
    "table",
    "file",
    "file1",
    "table_flush_header",
    "table_flush_line",
    "get_line_number_file",
]
